@@ -29,10 +29,12 @@
 
 #include <pcap.h>
 
+#include <osmocom/core/osmo_io.h>
 #include <osmocom/core/select.h>
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/write_queue.h>
 #include <osmocom/core/socket.h>
+#include <osmocom/netif/stream.h>
 
 #include <osmo-pcap/wireformat.h>
 #include <osmo-pcap/osmo_tls.h>
@@ -72,6 +74,32 @@ enum {
 	SERVER_CTR_NOCLIENT,
 };
 
+struct osmo_pcap_wr_file;
+typedef void (*osmo_pcap_wr_file_flush_completed_cb_t)(struct osmo_pcap_wr_file *wrf, void *data);
+struct osmo_pcap_wr_file {
+	struct llist_head entry; /* entry into (osmo_pcap_conn)->wrf_flushing_list */
+	void *data; /* user backpointer */
+	/* canonicalized absolute pathname of pcap file we write to */
+	char *filename;
+	/* file descriptor of the file we write to */
+	struct osmo_io_fd *local_iofd;
+	/* Current write offset of the file we write to (local_fd) */
+	off_t wr_offset;
+	/* Number of bytes confirmed to be written, <=wr_offset */
+	off_t wr_completed;
+	osmo_pcap_wr_file_flush_completed_cb_t flush_completed_cb;
+};
+struct osmo_pcap_wr_file *osmo_pcap_wr_file_alloc(void *ctx, void *data);
+void osmo_pcap_wr_file_free(struct osmo_pcap_wr_file *wrf);
+void osmo_pcap_wr_file_set_flush_completed_cb(struct osmo_pcap_wr_file *wrf, osmo_pcap_wr_file_flush_completed_cb_t flush_completed_cb);
+int osmo_pcap_wr_file_open(struct osmo_pcap_wr_file *wrf, const char *filename, mode_t mode);
+void osmo_pcap_wr_file_close(struct osmo_pcap_wr_file *wrf);
+int osmo_pcap_wr_file_write_msgb(struct osmo_pcap_wr_file *wrf, struct msgb *msg);
+bool osmo_pcap_wr_file_has_pending_writes(const struct osmo_pcap_wr_file *wrf);
+int osmo_pcap_wr_file_flush(struct osmo_pcap_wr_file *wrf, struct llist_head *wrf_flushing_list);
+bool osmo_pcap_wr_file_is_flushing(const struct osmo_pcap_wr_file *wrf);
+void osmo_pcap_wr_file_move_to_dir(struct osmo_pcap_wr_file *wrf, const char *dst_dirpath);
+
 struct osmo_pcap_conn {
 	/* list of connections */
 	struct llist_head entry;
@@ -84,27 +112,23 @@ struct osmo_pcap_conn {
 	struct osmo_sockaddr rem_addr;
 
 	/* Remote connection */
-	struct osmo_wqueue rem_wq;
-	int local_fd;
-	/* canonicalized absolute pathname of pcap file we write to */
-	char *curr_filename;
-	/* Current write offset of the file we write to (local_fd) */
-	off_t wr_offset;
+	struct osmo_stream_srv *srv;
+	struct osmo_pcap_wr_file *wrf;
+	/* list of osmo_pcap_wr_file->entry.
+	 * wrf which we want to close but still have pending writes to be completed */
+	struct llist_head wrf_flushing_list;
 
 	/* pcap stuff */
 	enum osmo_pcap_fmt file_fmt;
 	bool pcapng_endian_swapped;
-	uint8_t *file_hdr;
-	uint32_t file_hdr_len;
+	struct msgb *file_hdr_msg;
 
 	/* last time */
 	struct tm last_write;
 
 	/* read buffering */
 	int state;
-	int pend;
 	bool reopen_delayed;
-	struct osmo_pcap_data *data;
 	size_t data_max_len; /* size of allocated buffer in data->data. */
 
 	/* statistics */
@@ -112,15 +136,16 @@ struct osmo_pcap_conn {
 
 	/* tls */
 	bool tls_use;
-	bool direct_read;
 	size_t tls_limit_read;
 	struct osmo_tls_session tls_session;
+	struct osmo_wqueue rem_wq;
+	struct msgb *rx_tls_dec_msg; /* Used to store TLS decoded data */
 };
 
 void osmo_pcap_conn_free(struct osmo_pcap_conn *conn);
 void vty_server_init(void);
 void osmo_pcap_conn_close(struct osmo_pcap_conn *conn);
-int osmo_pcap_conn_process_data(struct osmo_pcap_conn *conn, const uint8_t *data, size_t len);
+int osmo_pcap_conn_process_data(struct osmo_pcap_conn *conn, struct msgb *msg);
 void osmo_pcap_conn_restart_trace(struct osmo_pcap_conn *conn);
 void osmo_pcap_conn_close_trace(struct osmo_pcap_conn *conn);
 void osmo_pcap_conn_event(struct osmo_pcap_conn *conn,
@@ -131,7 +156,7 @@ struct osmo_pcap_server {
 
 	int port;
 	char *addr;
-	struct osmo_fd listen_fd;
+	struct osmo_stream_srv_link *srv_link;
 
 	/* zeromq handling */
 	int zmq_port;
@@ -174,7 +199,7 @@ extern struct osmo_pcap_server *pcap_server;
 extern const struct rate_ctr_group_desc pcap_peer_group_desc;
 extern const struct rate_ctr_group_desc pcap_server_group_desc;
 struct osmo_pcap_server *osmo_pcap_server_alloc(void *ctx);
-
+void osmo_pcap_server_free(struct osmo_pcap_server *psrv);
 void osmo_pcap_server_reopen(struct osmo_pcap_server *server);
 int osmo_pcap_server_listen(struct osmo_pcap_server *server);
 struct osmo_pcap_conn *osmo_pcap_server_find_or_create(struct osmo_pcap_server *ser,
