@@ -96,8 +96,9 @@ static int config_write_server(struct vty *vty)
 
 	vty_out(vty, "server%s", VTY_NEWLINE);
 
-	if (pcap_server->base_path)
-		vty_out(vty, " base-path %s%s", pcap_server->base_path, VTY_NEWLINE);
+	vty_out(vty, " base-path %s%s", pcap_server->base_path, VTY_NEWLINE);
+	if (pcap_server->completed_path)
+		vty_out(vty, " completed-path %s%s", pcap_server->completed_path, VTY_NEWLINE);
 	vty_out(vty, " file-permission-mask 0%o%s", pcap_server->permission_mask, VTY_NEWLINE);
 	if (pcap_server->addr)
 		vty_out(vty, " server ip %s%s", pcap_server->addr, VTY_NEWLINE);
@@ -127,7 +128,7 @@ static int config_write_server(struct vty *vty)
 	llist_for_each_entry(conn, &pcap_server->conn, entry) {
 		vty_out(vty, " client %s %s%s%s%s",
 			conn->name, conn->remote_host,
-			conn->no_store ? " no-store" : " store",
+			conn->store ? " store" : " no-store",
 			conn->tls_use ? " tls" : "",
 			VTY_NEWLINE);
 	}
@@ -149,8 +150,39 @@ DEFUN(cfg_server_base,
       "base-path PATH",
       "Base path for log files\n" "Path\n")
 {
-	talloc_free(pcap_server->base_path);
-	pcap_server->base_path = talloc_strdup(pcap_server, argv[0]);
+	/* Validate we can resolve path: */
+	char *tmp = realpath(argv[0], NULL);
+	if (!tmp) {
+		vty_out(vty, "%% Failed to resolve path '%s': %s%s", argv[0], strerror(errno), VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+	free(tmp);
+	osmo_talloc_replace_string(pcap_server, &pcap_server->base_path, argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_server_no_completed_path,
+      cfg_server_no_completed_path_cmd,
+      "no completed-path",
+      NO_STR "Base path for completed (already closed, rotated) log files. Completed files won't be moved.\n")
+{
+	TALLOC_FREE(pcap_server->completed_path);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_server_completed_path,
+      cfg_server_completed_path_cmd,
+      "completed-path PATH",
+      "Base path for completed (already closed, rotated) log files\n" "Path\n")
+{
+	/* Validate we can resolve path: */
+	char *tmp = realpath(argv[0], NULL);
+	if (!tmp) {
+		vty_out(vty, "%% Failed to resolve path '%s': %s%s", argv[0], strerror(errno), VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+	free(tmp);
+	osmo_talloc_replace_string(pcap_server, &pcap_server->completed_path, argv[0]);
 	return CMD_SUCCESS;
 }
 
@@ -319,30 +351,30 @@ DEFUN(cfg_server_max_snaplen,
 static int manage_client(struct osmo_pcap_server *pcap_server,
 			struct vty *vty,
 			const char *name, const char *remote_host,
-			bool no_store, bool use_tls)
+			bool store, bool use_tls)
 {
 	struct osmo_pcap_conn *conn;
-	conn = osmo_pcap_server_find(pcap_server, name);
+	conn = osmo_pcap_server_find_or_create(pcap_server, name);
 	if (!conn) {
 		vty_out(vty, "Failed to create a pcap server.\n");
 		return CMD_WARNING;
 	}
 
-	talloc_free(conn->remote_host);
-	conn->remote_host = talloc_strdup(pcap_server, remote_host);
-	inet_aton(remote_host, &conn->remote_addr);
+	if (osmo_sockaddr_from_str_and_uint(&conn->rem_addr, remote_host, 0) < 0) {
+		vty_out(vty, "Failed parsing address\n");
+		return CMD_WARNING;
+	}
+	osmo_talloc_replace_string(pcap_server, &conn->remote_host, remote_host);
 
-	/* Checking no-store and maybe closing a pcap file */
-	if (no_store) {
-		osmo_pcap_server_close_trace(conn);
-		conn->no_store = 1;
-	} else
-		conn->no_store = 0;
+	/* Checking store and maybe closing a pcap file */
+	if (!store)
+		osmo_pcap_conn_close_trace(conn);
+	conn->store = store;
 
 	if (use_tls) {
 		/* force moving to TLS */
 		if (!conn->tls_use)
-			osmo_pcap_server_close_conn(conn);
+			osmo_pcap_conn_close(conn);
 		conn->tls_use = true;
 	} else {
 		conn->tls_use = false;
@@ -351,25 +383,20 @@ static int manage_client(struct osmo_pcap_server *pcap_server,
 	return CMD_SUCCESS;
 }
 
-
 DEFUN(cfg_server_client,
       cfg_server_client_cmd,
-      "client NAME A.B.C.D [no-store] [tls]",
-      CLIENT_STR "Remote name used in filenames\n"
-      "IP of the remote\n" "Do not store traffic\n"
+      "client NAME A.B.C.D [(store|no-store)] [tls]",
+      CLIENT_STR
+      "Remote name used in filenames\n"
+      "IP of the remote\n"
+      "Store traffic\n" "Do not store traffic\n"
       "Use Transport Level Security\n")
 {
-	return manage_client(pcap_server, vty, argv[0], argv[1], argc >= 3, argc >= 4);
-}
-
-DEFUN(cfg_server_client_store_tls,
-      cfg_server_client_store_tls_cmd,
-      "client NAME A.B.C.D store [tls]",
-      CLIENT_STR "Remote name used in filenames\n"
-      "IP of the remote\n" "Do not store traffic\n"
-      "Use Transport Level Security\n")
-{
-	return manage_client(pcap_server, vty, argv[0], argv[1], false, argc >= 3);
+	bool store = true;
+	if (argc >= 3 && strcmp(argv[2], "no-store") == 0)
+		store = false;
+	bool tls = argc >= 4;
+	return manage_client(pcap_server, vty, argv[0], argv[1], store, tls);
 }
 
 DEFUN(cfg_server_no_client,
@@ -378,13 +405,13 @@ DEFUN(cfg_server_no_client,
       NO_STR CLIENT_STR "The name\n")
 {
 	struct osmo_pcap_conn *conn;
-	conn = osmo_pcap_server_find(pcap_server, argv[0]);
+	conn = osmo_pcap_server_find_or_create(pcap_server, argv[0]);
 	if (!conn) {
 		vty_out(vty, "Failed to create a pcap server.\n");
 		return CMD_WARNING;
 	}
 
-	osmo_pcap_server_delete(conn);
+	osmo_pcap_conn_free(conn);
 	return CMD_SUCCESS;
 }
 
@@ -666,6 +693,8 @@ void vty_server_init(void)
 	install_node(&server_node, config_write_server);
 
 	install_element(SERVER_NODE, &cfg_server_base_cmd);
+	install_element(SERVER_NODE, &cfg_server_no_completed_path_cmd);
+	install_element(SERVER_NODE, &cfg_server_completed_path_cmd);
 	install_element(SERVER_NODE, &cfg_server_file_permission_mask_cmd);
 	install_element(SERVER_NODE, &cfg_server_ip_cmd);
 	install_element(SERVER_NODE, &cfg_server_port_cmd);
@@ -700,6 +729,5 @@ void vty_server_init(void)
 	install_element(SERVER_NODE, &cfg_tls_dh_pkcs3_cmd);
 
 	install_element(SERVER_NODE, &cfg_server_client_cmd);
-	install_element(SERVER_NODE, &cfg_server_client_store_tls_cmd);
 	install_element(SERVER_NODE, &cfg_server_no_client_cmd);
 }
